@@ -1,4 +1,5 @@
-#!/usr/bin/perl -I/home/phil/z/perl/cpan/DataTableText/lib
+#!/usr/bin/perl
+#-I/home/phil/z/perl/cpan/DataTableText/lib
 #-------------------------------------------------------------------------------
 # Lint xml files in parallel using xmllint and report the failure rate
 # Philip R Brenan at gmail dot com, Appa Apps Ltd, 2016
@@ -11,7 +12,8 @@ use warnings FATAL => qw(all);
 use strict;
 use Carp;
 use Data::Table::Text qw(:all);
-our $VERSION = 2017.715;
+use Digest::SHA qw(sha256_hex);
+our $VERSION = 2017.716;
 
 #1 Constructor                                                                  # Construct a new linter
 
@@ -28,6 +30,8 @@ genLValueScalarMethods(qw(dtds));
 genLValueScalarMethods(qw(errors));                                             # Number of lint errors detected by xmllint
 genLValueScalarMethods(qw(linted));                                             # Date the lint was performed
 genLValueScalarMethods(qw(project));                                            # Optional project name to allow error counts to be aggregated by project
+genLValueScalarMethods(qw(processes));                                          # Maximum number of lint  processes to run in parallel - 8 by default
+genLValueScalarMethods(qw(sha256));                                             # String containing the xml to be written or the xml read
 genLValueScalarMethods(qw(source));                                             # String containing the xml to be written or the xml read
 
 #1 Lint                                                                         # Lint xml files in parallel
@@ -36,6 +40,12 @@ my @pids;
 
 sub lint($@)                                                                    # Store some xml in a file and apply xmllint in parallel
  {my ($lint, %attributes) = @_;                                                 # Linter, attributes to be recorded as xml comments
+
+  if (1)                                                                        # Maximum amount of parallelism
+   {my $processes = $lint->processes // 8;                                      # Maximum number of processes
+    waitpid(pop @pids, 0) while @pids > $processes;                             # Wait until enough sub processes have completed
+   }
+
   if (my $pid = fork())                                                         # Perform lints in parallel
    {push @pids, $pid;
     return;
@@ -55,6 +65,7 @@ sub lint($@)
   $attributes{catalog} = $C if $C;
   $attributes{dtds}    = $d if $d;
   $attributes{project} = $P if $P;
+  $attributes{sha256}  = sha256_hex($lint->source);                             # Digest of text
 
   my $a = sub                                                                   # Attributes to be recorded with the xml
    {my @s;
@@ -109,7 +120,7 @@ sub read($)
 
 sub wait()                                                                      # Wait for all lints to finish
 
- {waitpid($_, 0) for @pids;                                                     # Wait until sub processes have completed
+ {waitpid(pop @pids, 0) while @pids;                                            # Wait until sub processes have completed
  }
 
 sub clear($@)                                                                   # Clear the results of a prior run
@@ -189,13 +200,15 @@ END
                   grep {$files{$_} > 0}
                   keys %files;
 
-  push @report, <<END;
+  if (my $filesFail = @filesFail)                                               # Failing files report
+   {push @report, <<END;
 
-FailingFiles
+$filesFail FailingFiles
    #  Errors  Project       File
 END
-  for(1..@filesFail)
-   {push @report, sprintf("%4d  %6d  %-12.12s  %s\n", $_, @{$filesFail[$_-1]});
+    for(1..@filesFail)
+     {push @report, sprintf("%4d  %6d  %-12.12s  %s\n", $_,@{$filesFail[$_-1]});
+     }
    }
 
   return bless                                                                  # Return report
@@ -204,7 +217,7 @@ END
     numberOfProjects =>$numberOfProjects,
     numberOfFiles    =>$numberOfFiles,
     failingFiles     =>[@filesFail],
-    print            =>(join "\n", @report),
+    print            =>(join '', @report),
    }, 'Data::Edit::Xml::Lint::Report';
  }
 
@@ -302,6 +315,7 @@ Produces:
     4       1  bbb           out/bbb7.xml
     5       1  aaa           out/aaa3.xml
 
+
 =head1 Description
 
 =head2 Constructor
@@ -317,37 +331,47 @@ Create a new xml linter - call this method statically as in Data::Edit::Xml::Lin
 
 Attributes describing a lint
 
-=head4 file
+=head4 file :lvalue
 
 File that the xml will be written to and read from
 
 
-=head4 catalog
+=head4 catalog :lvalue
 
 Optional catalog file containing the locations of the DTDs used to validate the xml
 
 
-=head4 dtds
+=head4 dtds :lvalue
 
 Optional directory containing the DTDs used to validate the xml
 
 
-=head4 errors
+=head4 errors :lvalue
 
 Number of lint errors detected by xmllint
 
 
-=head4 linted
+=head4 linted :lvalue
 
 Date the lint was performed
 
 
-=head4 project
+=head4 project :lvalue
 
 Optional project name to allow error counts to be aggregated by project
 
 
-=head4 source
+=head4 processes :lvalue
+
+Maximum number of lint  processes to run in parallel - 8 by default
+
+
+=head4 sha256 :lvalue
+
+String containing the xml to be written or the xml read
+
+
+=head4 source :lvalue
 
 String containing the xml to be written or the xml read
 
@@ -398,32 +422,32 @@ Analyse the results of prior lints and return a hash reporting various statistic
 
 =head3 Attributes
 
-=head4 passRatePercent
+=head4 passRatePercent :lvalue
 
 Total number of passes as a percentage of all input files
 
 
-=head4 timestamp
+=head4 timestamp :lvalue
 
 Timestamp of report
 
 
-=head4 numberOfProjects
+=head4 numberOfProjects :lvalue
 
 Number of projects defined - each project can contain zero or more files
 
 
-=head4 numberOfFiles
+=head4 numberOfFiles :lvalue
 
 Number of files encountered
 
 
-=head4 failingFiles
+=head4 failingFiles :lvalue
 
 Array of [number of errors, project, file] ordered from least to most errors
 
 
-=head4 report
+=head4 print :lvalue
 
 A printable report of the above
 
@@ -456,12 +480,18 @@ L<numberOfProjects|/numberOfProjects>
 
 L<passRatePercent|/passRatePercent>
 
+L<print|/print>
+
+L<processes|/processes>
+
 L<project|/project>
 
 L<read|/read>
 
 L<report|/report>
 
+L<sha256|/sha256>
+
 L<source|/source>
 
 L<timestamp|/timestamp>
@@ -495,12 +525,14 @@ under the same terms as Perl itself.
 
 =cut
 # podDocumentation
+# pod2html --infile=lib/Data/Edit/Xml/Lint.pm --outfile=zzz.html
 
 __DATA__
 use warnings FATAL=>qw(all);
 use strict;
 use Test::More tests=>33;
-#Test::More->builder->output("/dev/null");                                       # Show only errors during testing - but this must be commented out for production
+
+#Test::More->builder->output("/dev/null");                                      # Show only errors during testing - but this must be commented out for production
 
 my $outDir = "out";                                                             # Output directory
 
